(*********************************************
TSpeedButtonRollOver -> TSpeedButton

A TSpeedButton that supports different glyphs for
3 states, up, down, and mouse over.  You supply a
seperate Glyph for each of these states.

!! IMPORTANT NOTE !!

In order for the Glyphs to be assigned properly when the
mouse moves off a button, you must call the provided
SpeedButtonRollOverMouseMove procedure on the MouseMove
event of the form or control containing the speed buttons.

Properties:

GlyphDown:
  The glyph that will be used when the button is pressed.

GlyphOver:
  The glyph that will be used when the mouse passes over the
  button.

GlyphUp:
  The glyph that will be used when the button is not pressed.
*********************************************)

unit SpeedBut;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons;

type
  TSpeedButtonRollOver = class( TSpeedButton )
  private
     bPushing: boolean;
     FGlyphUp: TBitmap;
     FGlyphDown: TBitmap;
     FGlyphOver: TBitmap;
     FGlyphSet: TBitmap;
  protected
     procedure Loaded; override;
     procedure MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer ); override;
     procedure MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: Integer ); override;
     procedure MouseMove( Shift: TShiftState; X, Y: Integer ); override;
     procedure SetGlyphUp( bm: TBitmap );
     procedure SetGlyphDown( bm: TBitmap );
     procedure SetGlyphOver( bm: TBitmap );
  public
     constructor Create( AOwner: TComponent ); override;
     destructor Destroy; override;
  published
     property GlyphUp: TBitmap read FGlyphUp write SetGlyphUp;
     property GlyphDown: TBitmap read FGlyphDown write SetGlyphDown;
     property GlyphOver: TBitmap read FGlyphOver write SetGlyphOver;
  end;

procedure Register;
procedure SpeedButtonRollOverMouseMove;

implementation

const
  spdLast: TSpeedButtonRollOver = nil;

procedure SpeedButtonRollOverMouseMove;
begin
  if spdLast <> nil then
     begin
        spdLast.Glyph := spdLast.GlyphUp;
        spdLast.FGlyphSet := spdLast.GlyphUp;
        spdLast := nil;
     end;
end;

constructor TSpeedButtonRollOver.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  FGlyphUp := TBitmap.Create;
  FGlyphDown := TBitmap.Create;
  FGlyphOver := TBitmap.Create;
  Layout := blGlyphTop;
end;

destructor TSpeedButtonRollOver.Destroy;
begin
  FGlyphUp.Free;
  FGlyphDown.Free;
  FGlyphOver.Free;
  inherited Destroy;
end;

procedure TSpeedButtonRollOver.SetGlyphUp( bm: TBitmap );
begin
  FGlyphUp.Assign( bm );
  Glyph := FGlyphUp;
end;

procedure TSpeedButtonRollOver.SetGlyphDown( bm: TBitmap );
begin
  FGlyphDown.Assign( bm );
end;

procedure TSpeedButtonRollOver.SetGlyphOver( bm: TBitmap );
begin
  FGlyphOver.Assign( bm );
end;

procedure TSpeedButtonRollOver.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer );
begin
  bPushing := TRUE;
  Glyph := GlyphDown;
  FGlyphSet := GlyphDown;
  inherited MouseDown( Button, Shift, X, Y );
end;

procedure TSpeedButtonRollOver.MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  bPushing := FALSE;
  Glyph := GlyphUp;
  FGlyphSet := GlyphUp;
  inherited MouseUp( Button, Shift, X, Y );
end;

procedure TSpeedButtonRollOver.MouseMove( Shift: TShiftState; X, Y: Integer );
begin
  inherited MouseMove( Shift, X, Y );

  if ( spdLast <> nil ) and ( spdLast <> self ) then
     begin
        spdLast.Glyph := spdLast.GlyphUp;
        spdLast.FGlyphSet := spdLast.GlyphUp;
     end;

  if not bPushing then
     if FGlyphSet <> GlyphOver then
        begin
           Glyph := GlyphOver;
           FGlyphSet := GlyphOver;
        end;

  spdLast := self;
end;

procedure TSpeedButtonRollOver.Loaded;
begin
  inherited Loaded;
  Glyph := GlyphUp;
  FGlyphSet := GlyphUp;
end;

procedure Register;
begin
  RegisterComponents( 'Vis', [TSpeedButtonRollOver] );
end;

end.
